-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Aloft")
local SML = AceLibrary("SharedMedia-1.0")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

Aloft = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0", "FuBarPlugin-2.0")
Aloft.version = "1.0"
Aloft.hasIcon = "Interface\\Icons\\Spell_Frost_Wisp"	-- Placeholder
Aloft.defaultPosition = 'LEFT'
Aloft.hideWithoutStandby = true
Aloft.independentProfile = true

-----------------------------------------------------------------------------

function Aloft:FormatLargeValue(value)
	if value < 10000 then
		-- Force it to an integer
		return string.format("%d", value)
	elseif value < 100000 then
		return string.format("%.1fk", value / 1000)
	elseif value < 1000000 then
		return string.format("%dk", value / 1000)
	else
		return string.format("%.2fm", value / 1000000)
	end
end

-----------------------------------------------------------------------------

local numberOfWorldChildren = 0
local nameplateList = { }
local visibleNameplateList = { }
local outlineLookup = { ["None"] = nil, ["Normal"] = "OUTLINE", ["Thick"] = "THICKOUTLINE" }

-----------------------------------------------------------------------------

function Aloft:OnInitialize()
	self:SetText(L["Aloft"])
	SML:Register("statusbar", "Aloft:Simple Border", "Interface\\Addons\\Aloft\\Images\\SimpleBorder")
end

function Aloft:OnEnable()
	self:ScheduleRepeatingEvent("AloftUpdate", self.Update, 0.2, self)
	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateCastBars()
			self:UpdateCastBarOverlays()
		end
	end)
	
	self.HideNameplates = HideNameplates
	self.ShowNameplates = ShowNameplates
	self.HideFriendNameplates = HideFriendNameplates
	self.ShowFriendNameplates = ShowFriendNameplates
	HideNameplates = function() self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.HideNameplates() end
	ShowNameplates = function() self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.ShowNameplates() end
	HideFriendNameplates = function() self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.HideFriendNameplates() end
	ShowFriendNameplates = function() self:ScheduleEvent("AloftReportNameplateStatus", self.ReportNameplateStatus, 0, self) self.ShowFriendNameplates() end
end

function Aloft:OnDisable()
end

function Aloft:ReportNameplateStatus()
	if self.db.profile.reportStatusChanges then
		self:Print((NAMEPLATES_ON and L["HostileNameplatesEnabled"] or L["HostileNameplatesDisabled"])..
					(FRIENDNAMEPLATES_ON and L["FriendlyNameplatesEnabled"] or L["FriendlyNameplatesDisabled"]))
	end
end

function Aloft:OnClick()
	if NAMEPLATES_ON then
		HideNameplates()
		NAMEPLATES_ON = false
	else
		ShowNameplates()
		NAMEPLATES_ON = true
	end
end

function Aloft:IsNameplateFrame(frame)
	if frame:GetName() then
		return false
	end

	local overlayFrame = frame:GetRegions()
	if not overlayFrame or overlayFrame:GetObjectType() ~= "Texture" or overlayFrame:GetTexture() ~= "Interface\\Tooltips\\Nameplate-Border" then
		return false
	end
	
	return true
end

function Aloft:ProcessChildFrames(...)
	for i = 1, select('#', ...) do
		local frame = select(i, ...)
		if self:IsNameplateFrame(frame) then 
			if not nameplateList[frame] then
				self:SetupNamePlate(frame)
				nameplateList[frame] = frame
				visibleNameplateList[frame] = frame
			end
		end
	end
end

function Aloft:Update()
	local newNumberOfWorldChildren = WorldFrame:GetNumChildren()
	if newNumberOfWorldChildren ~= numberOfWorldChildren then
		numberOfWorldChildren = newNumberOfWorldChildren

		self:ProcessChildFrames(WorldFrame:GetChildren())
	end
	
	for frame in pairs(visibleNameplateList) do	
		local aloftData = frame.aloftData
		local healthBar = aloftData.healthBar
		local r, g, b, a = healthBar:GetStatusBarColor()

		-- An awful awful update routine to stop the game overwriting our colors sporadically
		if r ~= aloftData.healthBarR or g ~= aloftData.healthBarG or b ~= aloftData.healthBarB or a ~= aloftData.healthBarA then
			self:DetermineNameplateType(aloftData, healthBar)
			self:TriggerEvent("Aloft:OnHealthBarColorChanged", frame)
		end
		
		local nameFrame = select(5, frame:GetRegions())
		r, g, b = nameFrame:GetTextColor()
		r, g, b = floor(r * 256), floor(g*256), floor(b*256)
		if r == 255 and g == 0 and b == 0 then
			if not frame.aloftData.recentlyDamaged then
				frame.aloftData.recentlyDamaged = true
				self:TriggerEvent("Aloft:OnRecentlyDamagedChanged", frame)
			end
		else
			if frame.aloftData.recentlyDamaged then
				frame.aloftData.recentlyDamaged = false
				self:TriggerEvent("Aloft:OnRecentlyDamagedChanged", frame)
			end
		end
	end
end

function Aloft:IterateNameplates()
	return pairs(nameplateList)
end

function Aloft:IterateVisibleNameplates()
	return pairs(visibleNameplateList)
end

-----------------------------------------------------------------------------

function Aloft:GetTargetNameplateFrame()
	if not UnitExists("target") then return nil end
	
	for frame in pairs(visibleNameplateList) do
		if frame:GetAlpha() == 1 then
			return frame
		end
	end
	return nil
end

function Aloft:GetNameplateFrame(unitName)
	for frame in pairs(visibleNameplateList) do
		if frame.aloftData.name == unitName then
			return frame
		end
	end
	return nil
end

function Aloft:GetUnitNameplateFrame(unitid)
	if not UnitExists(unitid) then
		return nil 
	end
	return self:GetNameplateFrame(UnitName(unitid))
end

-----------------------------------------------------------------------------

function Aloft:PrepareText(textFrame, data)
	textFrame:SetFont(SML:Fetch("font", data.font), data.fontSize, outlineLookup[data.outline])
	if data.shadow then
		textFrame:SetShadowOffset(1, -1)
	else
		textFrame:SetShadowOffset(0, 0)
	end
	if data.alignment then
		textFrame:SetJustifyH(data.alignment)
	end
	if data.color then
		textFrame:SetTextColor(unpack(data.color))
	end
end

function Aloft:PlaceFrame(frame, relativeFrame, data)
	if data.enable then
		frame:ClearAllPoints()
		frame:SetPoint(data.point, relativeFrame, data.relativeToPoint, data.offsetX, data.offsetY)
		
		if data.size then
			frame:SetWidth(data.size)
			frame:SetHeight(data.size)
		end
		frame:SetAlpha(data.alpha or 1.0)
	else
		frame:SetAlpha(0)
	end
end

-----------------------------------------------------------------------------

function Aloft:OnHealthBarValueChanged(frame, healthBar)
	if frame:IsShown() then
		local aloftData = frame.aloftData
		local overlayFrame, _, _, _, nameFrame = frame:GetRegions()
		aloftData.healthBarValue = healthBar:GetValue()
		
		if aloftData.nameFrameText ~= nameFrame:GetText() then
			self:OnNameplateShow(frame)
		else
			self:TriggerEvent("Aloft:OnHealthBarValueChanged", frame)
		end
	end
end

function Aloft:DetermineNameplateType(aloftData, healthBar)
	local r, g, b, a = healthBar:GetStatusBarColor()
	r, g, b = floor(r * 256), floor(g*256), floor(b*256)

	if r == 255 and g == 0 and b == 0 then
		if aloftData.isBoss then
			aloftData.type = "hostileBoss"
		else
			aloftData.type = "hostile"
		end
	elseif r == 0 and g == 0 and b == 255 then
		aloftData.type = "friendlyPlayer"
	elseif r == 0 and g == 255 and b == 0 then
		if aloftData.isBoss then
			aloftData.type = "friendlyBoss"
		else
			if aloftData.unitid then
				-- Must be a group pet
				aloftData.type = "groupPet"
			else
				aloftData.type = "friendlyNPC"
			end
		end
	elseif r == 255 and g == 255 and b == 0 then
		aloftData.type = "neutral"
	else
--		ChatFrame3:AddMessage("rgb = "..r.."."..g.."."..b)
	end
end

function Aloft:DetermineRecentlyDamaged(aloftData, nameFrame)
	local r, g, b = nameFrame:GetTextColor()
	r, g, b = floor(r * 256), floor(g*256), floor(b*256)
	if r == 255 and g == 0 and b == 0 then
		aloftData.recentlyDamaged = true
	else
		aloftData.recentlyDamaged = false
	end
end

function Aloft:OnNameplateShow(frame)
	-- When a nameplate is first shown, grab some data about it
	local _, _, _, _, nameFrame, levelTextFrame, bossIconFrame = frame:GetRegions()
	local aloftData = frame.aloftData
	local healthBar = aloftData.healthBar

	if aloftData.nameFrameText ~= nameFrame:GetText() then
		aloftData.name = nameFrame:GetText()
		aloftData.nameFrameText = name
	else
		aloftData.nameFrameText = nameFrame:GetText()
	end
	aloftData.unitid = roster:GetUnitIDFromName(aloftData.name)
	aloftData.healthBarValue = healthBar:GetValue()
	aloftData.isBoss = bossIconFrame:IsShown()
	aloftData.level = tonumber(levelTextFrame:GetText()) or -1
	if not levelTextFrame:IsShown() then
		aloftData.level = -1
	end

	self:DetermineRecentlyDamaged(aloftData, nameFrame)
	self:DetermineNameplateType(aloftData, healthBar)

--	ChatFrame3:AddMessage("name: "..aloftData.name.." | unitid: "..tostring(aloftData.unit).." | nameFrame: "..nameFrame:GetText().." | level: "..aloftData.level)

	self:TriggerEvent("Aloft:OnNameplateShow", frame)
end

function Aloft:OnNameplateHide(frame)
	frame.aloftData.name = "[invalid]"
	frame.aloftData.nameFrameText = nil
	frame.aloftData.type = "unknown"
	self:TriggerEvent("Aloft:OnNameplateHide", frame)
end

function Aloft:UpdateEmulateBackgroundCastBarOverlay(castBarOverlayFrame, healthBar, castBar)
	if self.db.profile.castBarOverlay.enable and self.db.profile.castBarOverlay.emulateBackground then
		local minValue, maxValue = castBar:GetMinMaxValues()

		-- Just as the cast bar disappears, both seem to be set to 0.
		if minValue ~= maxValue then
			local k = (castBar:GetValue() - minValue) / (maxValue - minValue)
			if k == 1 then
				castBarOverlayFrame:Hide()
			else		
				castBarOverlayFrame:Show()
				castBarOverlayFrame:ClearAllPoints()
				
				local castBarWidth = castBar:GetWidth() -- self.db.profile.healthBar.width + self.db.profile.castBar.offsets.right - self.db.profile.castBar.offsets.left			
				castBarOverlayFrame:SetPoint("TOPRIGHT", healthBar, "BOTTOMRIGHT", self.db.profile.castBar.offsets.right, self.db.profile.castBar.offsets.vertical)
				castBarOverlayFrame:SetWidth((1.0-k) * castBarWidth)
				castBarOverlayFrame:SetHeight(self.db.profile.castBar.height)
				castBarOverlayFrame:SetTexCoord(k, 1.0, 0.0, 1.0)
			end
		end
	end
end

function Aloft:OnCastBarValueChanged(frame, castBar)
	local _, castBarOverlayFrame = frame:GetRegions()
	castBarOverlayFrame:SetVertexColor(unpack(self.db.profile.castBarOverlay.color))

	if castBar:IsShown() then
		self:UpdateEmulateBackgroundCastBarOverlay(castBarOverlayFrame, frame.aloftData.healthBar, castBar)
		self:TriggerEvent("Aloft:OnCastBarValueChanged", frame)
	end
end

function Aloft:PlaceCastBarAndOverlayFrames(castBar, castBarOverlayFrame, healthBar)
	local profile = self.db.profile
	local verticalOffset = profile.castBar.offsets.vertical

	castBar:ClearAllPoints()
	castBar:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT", profile.castBar.offsets.left, verticalOffset)
	castBar:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.castBar.offsets.right, verticalOffset - profile.castBar.height)

	-- Trying to anchor the castbar overlay to castbar results in some very unexpected complete-game crashes (no error message, or anything)..
	castBarOverlayFrame:ClearAllPoints()
	castBarOverlayFrame:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT", profile.castBar.offsets.left-profile.castBarOverlay.horizontalPadding, verticalOffset+profile.castBarOverlay.verticalPadding)
	castBarOverlayFrame:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.castBar.offsets.right+profile.castBarOverlay.horizontalPadding, verticalOffset - profile.castBar.height - profile.castBarOverlay.verticalPadding)
end

function Aloft:OnCastBarShow(frame, castBar)
	local healthBar = frame.aloftData.healthBar
	local _, castBarOverlayFrame, spellIconFrame = frame:GetRegions()

--[[
	if self.db.profile.castBar.hideExpired then
		if not targetSpellName then
			ChatFrame3:AddMessage("Trying to hide expired...")
			castBar:Hide()	
			castBarOverlayFrame:Hide()	
			spellIconFrame:Hide()
			return
		end
	end
]]

	local profile = self.db.profile

	self:PlaceCastBarAndOverlayFrames(castBar, castBarOverlayFrame, healthBar)
	castBarOverlayFrame:SetVertexColor(unpack(profile.castBarOverlay.color))

	if not profile.castBarOverlay.enable then
		castBarOverlayFrame:Hide()
	end

	self:UpdateEmulateBackgroundCastBarOverlay(castBarOverlayFrame, healthBar, castBar)

	self:TriggerEvent("Aloft:OnCastBarShow", frame)
end

function Aloft:OnCastBarHide(frame)
	self:TriggerEvent("Aloft:OnCastBarHide", frame)
end

-----------------------------------------------------------------------------

local function OnNameplateShow()
	visibleNameplateList[this] = this
	Aloft:OnNameplateShow(this)
end

local function OnNameplateHide()
	Aloft:OnNameplateHide(this)
	visibleNameplateList[this] = nil
end

local function OnHealthBarValueChanged()
	Aloft:OnHealthBarValueChanged(this:GetParent(), this)
end

local function OnCastBarValueChanged()
	Aloft:OnCastBarValueChanged(this:GetParent(), this)
end

local function OnCastBarShow()
	Aloft:OnCastBarShow(this:GetParent(), this)
end

local function OnCastBarHide()
	Aloft:OnCastBarHide(this:GetParent())	
end

local function SetHealthBarColor(this, r, g, b, a)
	local aloftData = this:GetParent().aloftData

	aloftData.SetHealthBarColor(this, r, g, b, a)	
	
	-- We read it back, because rounding seems to kick in and prevent them from being exactly what we sent..
	aloftData.healthBarR, aloftData.healthBarG, aloftData.healthBarB, aloftData.healthBarA  = this:GetStatusBarColor()
end

local function SetNameFrameText(this, text)
	local aloftData = this:GetParent().aloftData

	aloftData.nameFrameText = text
	
	aloftData.SetNameFrameText(this, text)
end

-----------------------------------------------------------------------------

function Aloft:SetupNamePlate(frame)
	frame.aloftData = { type = "unknown", healthBarValue = 0, lastCastTimeValue = 0 }

	local _, castBarOverlayFrame, _, _, nameFrame = frame:GetRegions()
	local healthBar, castBar = frame:GetChildren()

	-- Because healthBar and castBar are used extensively, reference them now
	frame.aloftData.healthBar = healthBar
	frame.aloftData.castBar = castBar

	castBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.castBar.texture))

	castBarOverlayFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.castBarOverlay.texture))
	castBarOverlayFrame:SetVertexColor(unpack(self.db.profile.castBarOverlay.color))

	self:PlaceCastBarAndOverlayFrames(castBar, castBarOverlayFrame, healthBar)
	
	frame:SetScript("OnShow", OnNameplateShow)
	frame:SetScript("OnHide", OnNameplateHide)

	healthBar:SetScript("OnValueChanged", OnHealthBarValueChanged)
	frame.aloftData.SetHealthBarColor = healthBar.SetStatusBarColor
	healthBar.SetStatusBarColor = SetHealthBarColor
	
	castBar:SetScript("OnShow", OnCastBarShow)
	castBar:SetScript("OnHide", OnCastBarHide)
	castBar:SetScript("OnValueChanged", OnCastBarValueChanged)

	frame.aloftData.SetNameFrameText = nameFrame.SetText
	nameFrame.SetText = SetNameFrameText

	self:TriggerEvent("Aloft:SetupFrame", frame)

	self:OnNameplateShow(frame)
end

-----------------------------------------------------------------------------
-- Options methods. All of these need access to nameplateList
-----------------------------------------------------------------------------

function Aloft:UpdateCastBars()
	for frame in pairs(nameplateList) do
		local _, castBarOverlayFrame = frame:GetRegions()
		local healthBar = frame.aloftData.healthBar
		local castBar = frame.aloftData.castBar
		
		castBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.castBar.texture))

		self:PlaceCastBarAndOverlayFrames(castBar, castBarOverlayFrame, healthBar)
	end
end

function Aloft:UpdateCastBarOverlays()
	for frame in pairs(nameplateList) do
		local _, castBarOverlayFrame = frame:GetRegions()
		local healthBar = frame.aloftData.healthBar
		local castBar = frame.aloftData.castBar

		castBarOverlayFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.castBarOverlay.texture))

		self:PlaceCastBarAndOverlayFrames(castBar, castBarOverlayFrame, healthBar)
		castBarOverlayFrame:SetTexCoord(0.0, 1.0, 0.0, 1.0)
	end
end

-----------------------------------------------------------------------------

Aloft.modulePrototype.PrepareText = Aloft.PrepareText
Aloft.modulePrototype.PlaceFrame = Aloft.PlaceFrame

function Aloft.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

-----------------------------------------------------------------------------
