-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftBossIcon")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Boss Icon"] = true,
	["Boss icon placement options"] = true,
	["Enable"] = true,
	["Enable boss icon display on the nameplate"] = true,
	["Size"] = true,
	["Size in pixels of the boss icon"] = true,
	["X Offset"] = true,
	["X offset of the boss icon"] = true,
	["Y Offset"] = true,
	["Y offset of the boss icon"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the boss icon"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the boss icon"] = true,
	["Alpha"] = true,
	["Sets the boss icon alpha"] = true,	
} end)

-----------------------------------------------------------------------------

local AloftBossIcon = Aloft:NewModule("bossIcon", "AceEvent-2.0")
AloftBossIcon.db = Aloft:AcquireDBNamespace("bossIcon")
Aloft:RegisterDefaults("bossIcon", "profile", {
	enable			= true,
	alpha			= 1.0,
	point			= "CENTER",
	relativeToPoint	= "CENTER",
	offsetX			= 0,
	offsetY			= 0,
	size			= 12
})

Aloft.OnMenuRequest.args.bossIcon =
{
	type = 'group',
	name = L["Boss Icon"],
	desc = L["Boss icon placement options"],
	args =
	{
		enable = 
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable boss icon display on the nameplate"],
			get = function() return AloftBossIcon.db.profile.enable end,
			set = function(v) AloftBossIcon.db.profile.enable = v AloftBossIcon:UpdateAll() end
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the boss icon"],
			get = function() return AloftBossIcon.db.profile.point end,
			set = function(v) AloftBossIcon.db.profile.point = v AloftBossIcon:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the health bar to anchor the boss icon"],
			get = function() return AloftBossIcon.db.profile.relativeToPoint end,
			set = function(v) AloftBossIcon.db.profile.relativeToPoint = v AloftBossIcon:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		size =
		{
			type = 'range',
			name = L["Size"],
			desc = L["Size in pixels of the boss icon"],
			min = 4,
			max = 32,
			step = 1,
			get = function() return AloftBossIcon.db.profile.size end,
			set = function(v) AloftBossIcon.db.profile.size = v AloftBossIcon:UpdateAll() end
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the boss icon"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftBossIcon.db.profile.offsetX end,
			set = function(v) AloftBossIcon.db.profile.offsetX = v AloftBossIcon:UpdateAll() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the boss icon"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftBossIcon.db.profile.offsetY end,
			set = function(v) AloftBossIcon.db.profile.offsetY = v AloftBossIcon:UpdateAll() end
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the boss icon alpha"],
			min = 0,
			max = 1.0,
			step = 0.05,
			get = function() return AloftBossIcon.db.profile.alpha end,
			set = function(v) AloftBossIcon.db.profile.alpha = v AloftBossIcon:UpdateAll() end
		},
	},
}

-----------------------------------------------------------------------------

function AloftBossIcon:UpdateAll()
	for frame in Aloft:IterateNameplates() do	
		local bossIconFrame = select(7, frame:GetRegions())
		self:PlaceFrame(bossIconFrame, frame.aloftData.healthBar, self.db.profile)
	end
end

-----------------------------------------------------------------------------

function AloftBossIcon:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
end

function AloftBossIcon:SetupFrame(frame)
	local bossIconFrame = select(7, frame:GetRegions())
	self:PlaceFrame(bossIconFrame, frame.aloftData.healthBar, self.db.profile)
end

-----------------------------------------------------------------------------
