-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftSpellIcon")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Spell Icon"] = true,
	["Spell icon placement options"] = true,
	["Enable"] = true,
	["Enable spell icon display on the nameplate"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the spell icon"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the cast bar to anchor the spell icon"] = true,
	["Size"] = true,
	["Size in pixels of the spell icon"] = true,
	["X Offset"] = true,
	["X offset of the spell icon"] = true,
	["Y Offset"] = true,
	["Y offset of the spell icon"] = true,
} end)

-----------------------------------------------------------------------------

local AloftSpellIcon = Aloft:NewModule("spellIcon", "AceEvent-2.0")
AloftSpellIcon.db = Aloft:AcquireDBNamespace("spellIcon")
Aloft:RegisterDefaults("spellIcon", "profile", {
	enable			= true,
	point			= "RIGHT",
	relativeToPoint	= "LEFT",
	offsetX			= 0,
	offsetY			= 0,
	size			= 12
})

Aloft.OnMenuRequest.args.spellIcon =
{
	type = 'group',
	name = L["Spell Icon"],
	desc = L["Spell icon placement options"],
	args =
	{
		enable = 
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable spell icon display on the nameplate"],
			get = function() return AloftSpellIcon.db.profile.enable end,
			set = function(v) AloftSpellIcon.db.profile.enable = v AloftSpellIcon:UpdateAll() end
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the spell icon"],
			get = function() return AloftSpellIcon.db.profile.point end,
			set = function(v) AloftSpellIcon.db.profile.point = v AloftSpellIcon:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the cast bar to anchor the spell icon"],
			get = function() return AloftSpellIcon.db.profile.relativeToPoint end,
			set = function(v) AloftSpellIcon.db.profile.relativeToPoint = v AloftSpellIcon:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		size =
		{
			type = 'range',
			name = L["Size"],
			desc = L["Size in pixels of the spell icon"],
			min = 4,
			max = 32,
			step = 1,
			get = function() return AloftSpellIcon.db.profile.size end,
			set = function(v) AloftSpellIcon.db.profile.size = v AloftSpellIcon:UpdateAll() end
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the spell icon"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftSpellIcon.db.profile.offsetX end,
			set = function(v) AloftSpellIcon.db.profile.offsetX = v AloftSpellIcon:UpdateAll() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the spell icon"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftSpellIcon.db.profile.offsetY end,
			set = function(v) AloftSpellIcon.db.profile.offsetY = v AloftSpellIcon:UpdateAll() end
		},
	},
}

-----------------------------------------------------------------------------

function AloftSpellIcon:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		local spellIconFrame = select(3, frame:GetRegions())
		self:PlaceFrame(spellIconFrame, frame.aloftData.castBar, self.db.profile)
	end
end

-----------------------------------------------------------------------------

function AloftSpellIcon:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
end

function AloftSpellIcon:SetupFrame(frame)
	local spellIconFrame = select(3, frame:GetRegions())
	self:PlaceFrame(spellIconFrame, frame.aloftData.castBar, self.db.profile)
	frame.aloftData.spellIconFrame = spellIconFrame
end

function AloftSpellIcon:OnCastBarShow(frame)
	self:PlaceFrame(frame.aloftData.spellIconFrame, frame.aloftData.castBar, self.db.profile)
end

-----------------------------------------------------------------------------
