-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBarTimeText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Cast Bar Time Text"] = true,
	["Cast bar time text options"] = true,
	["Enable"] = true,
	["Show cast bar time text"] = true,
	["Font"] = true, 
	["Sets the font for cast bar time text"] = true,
	["Font Size"] = true,
	["Sets the font height of the cast bar time text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on cast bar time text"] = true,
	["Alignment"] = true,
	["Sets the alignment of the cast bar time text"] = true,
	["Outline"] = true,
	["Sets the font outline for cast bar time text"] = true,
	["Position"] = true,
	["Adjust cast bar time text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	["Color"] = true,
	["Sets the cast bar time text color"] = true,

} end)

-----------------------------------------------------------------------------

local castBarTimeTextFrame = nil

-----------------------------------------------------------------------------

local AloftCastBarTimeText = Aloft:NewModule("CastBarTimeText", "AceEvent-2.0")
AloftCastBarTimeText.db = Aloft:AcquireDBNamespace("castBarTimeText")
Aloft:RegisterDefaults("castBarTimeText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= true,
	alignment		= "RIGHT",
	outline			= "None",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	color			= { 1, 1, 1, 1 }
})

Aloft.OnMenuRequest.args.castBarTimeText =
{
	type = 'group',
	name = L["Cast Bar Time Text"],
	desc = L["Cast bar time text options"],
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.enable end,
			set = function(v) AloftCastBarTimeText.db.profile.enable = v AloftCastBarTimeText:Update() end
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.font end,
			set = function(v) AloftCastBarTimeText.db.profile.font = v AloftCastBarTimeText:Update() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the cast bar time text"],
			max = 16,
			min = 5,
			step = 1,
			get = function() return AloftCastBarTimeText.db.profile.fontSize end,
			set = function(value) AloftCastBarTimeText.db.profile.fontSize = value AloftCastBarTimeText:Update() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.shadow end,
			set = function(v) AloftCastBarTimeText.db.profile.shadow = v AloftCastBarTimeText:Update() end
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.alignment end,
			set = function(value) AloftCastBarTimeText.db.profile.alignment = value AloftCastBarTimeText:Update() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline for cast bar time text"],
			get = function() return AloftCastBarTimeText.db.profile.outline end,
			set = function(value) AloftCastBarTimeText.db.profile.outline = value AloftCastBarTimeText:Update() end,
			validate = {"None", "Normal", "Thick" },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust cast bar time text position"],
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.left end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.left = v AloftCastBarTimeText:Update() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.right end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.right = v AloftCastBarTimeText:Update() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBarTimeText.db.profile.offsets.vertical end,
					set = function(v) AloftCastBarTimeText.db.profile.offsets.vertical = v AloftCastBarTimeText:Update() end
				},
			},
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the cast bar time text color"],
			get = function() return unpack(AloftCastBarTimeText.db.profile.color) end,
			set = function(r, g, b, a) AloftCastBarTimeText.db.profile.color = { r, g, b, a } AloftCastBarTimeText:Update() end,
			hasAlpha = true
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBarTimeText:Update()
	self:CreateFrame()
	for frame in Aloft:IterateVisibleNameplates() do
		self:SetupFrame(frame)
	end
end

-----------------------------------------------------------------------------

function AloftCastBarTimeText:CreateFrame()
	if not self.db.profile.enable then
		if castBarTimeTextFrame then
			self:UnregisterAllEvents()
			castBarTimeTextFrame:Hide()
		end
		return
	end

	if not castBarTimeTextFrame then
		castBarTimeTextFrame = UIParent:CreateFontString()
	end
	
	self:PrepareText(castBarTimeTextFrame, self.db.profile)

	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnCastBarValueChanged", "UpdateText")
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")
	self:RegisterEvent("Aloft:OnCastBarHide", "OnCastBarHide")
end

-----------------------------------------------------------------------------

function AloftCastBarTimeText:OnEnable()	
	self:CreateFrame()
end

function AloftCastBarTimeText:SetupFrame(frame)
	if frame.aloftData.name == UnitName("target") and frame:GetAlpha() == 1 then
		local castBar = frame.aloftData.castBar
		castBarTimeTextFrame:ClearAllPoints()
		castBarTimeTextFrame:SetPoint("TOPLEFT", castBar, "TOPLEFT", self.db.profile.offsets.left, self.db.profile.offsets.vertical)
		castBarTimeTextFrame:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", self.db.profile.offsets.right, self.db.profile.offsets.vertical)
		castBarTimeTextFrame:SetParent(frame)
	end	
end

function AloftCastBarTimeText:UpdateText(frame)
	local castBar = frame.aloftData.castBar
	local minValue, maxValue = castBar:GetMinMaxValues()
	local value = castBar:GetValue()

	if value >= maxValue or value <= minValue then
		castBarTimeTextFrame:SetText("")
		frame.aloftData.lastCastTimeValue = 0
	else
		if value > frame.aloftData.lastCastTimeValue then
			castBarTimeTextFrame:SetText(string.format("%.2f", maxValue-value))
			frame.aloftData.lastCastTimeValue = value
		else
			castBarTimeTextFrame:SetText(string.format("%.2f", value-minValue))
			frame.aloftData.lastCastTimeValue = value
		end
	end
end

function AloftCastBarTimeText:OnCastBarShow(frame)
	local castBar = frame.aloftData.castBar
	local profile = self.db.profile

	frame.aloftData.lastCastTimeValue = castBar:GetValue()
	castBarTimeTextFrame:ClearAllPoints()
	castBarTimeTextFrame:SetPoint("TOPLEFT", castBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
	castBarTimeTextFrame:SetPoint("BOTTOMRIGHT", castBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
	castBarTimeTextFrame:SetParent(frame)
	castBarTimeTextFrame:Show()
end

function AloftCastBarTimeText:OnCastBarHide(frame)
	castBarTimeTextFrame:Hide()
end

-----------------------------------------------------------------------------
