-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftHealthText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Health Text"] = true,
	["Health text options"] = true,
	["Enable"] = true,
	["Show health text on nameplates"] = true,
	["Font"] = true,
	["Sets the font for health text"] = true,
	["Font Size"] = true,
	["Sets the font height of the health text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on health text"] = true,
	["Outline"] = true,
	["Sets the outline for health text"] = true,
	["Mode"] = true,
	["Choose what to show for health text"] = true,
	["Anchor"] = true, 
	["Sets the anchor for the health text"] = true,
	["Anchor To"] = true, 
	["Sets the relative point on the health bar to anchor the health text"] = true,
	["X Offset"] = true,
	["X offset of the health text"] = true,
	["Y Offset"] = true,
	["Y offset of the health text"] = true,
	["Alpha"] = true,
	["Sets the alpha of the health text"] = true,
	["Color"] = true,
	["Sets the health text color"] = true,
} end)

-----------------------------------------------------------------------------

local AloftHealthText = Aloft:NewModule("HealthText", "AceEvent-2.0")
AloftHealthText.db = Aloft:AcquireDBNamespace("healthText")
Aloft:RegisterDefaults("healthText", "profile", {
	enable			= false,
	alpha			= 1,
	point			= "RIGHT",
	relativeToPoint	= "RIGHT",
	offsetX			= 0,
	offsetY			= 0,
	font			= "Arial Narrow",
	fontSize		= 9,
	shadow			= false,
	outline			= "Normal",
	mode			= "Percent",
	color			= { 1, 1, 1 }
})

Aloft.OnMenuRequest.args.healthText =
{
	type = 'group',
	name = L["Health Text"],
	desc = L["Health text options"],
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show health text on nameplates"],
			get = function() return AloftHealthText.db.profile.enable end,
			set = function(v) AloftHealthText.db.profile.enable = v AloftHealthText:UpdateAll() end
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for health text"],
			get = function() return AloftHealthText.db.profile.font end,
			set = function(v) AloftHealthText.db.profile.font = v AloftHealthText:UpdateAll() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the health text"],
			max = 16,
			min = 5,
			step = 1,
			get = function() return AloftHealthText.db.profile.fontSize end,
			set = function(value) AloftHealthText.db.profile.fontSize = value AloftHealthText:UpdateAll() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on health text"],
			get = function() return AloftHealthText.db.profile.shadow end,
			set = function(v) AloftHealthText.db.profile.shadow = v AloftHealthText:UpdateAll() end
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the outline for health text"],
			get = function() return AloftHealthText.db.profile.outline end,
			set = function(value) AloftHealthText.db.profile.outline = value AloftHealthText:UpdateAll() end,
			validate = {"None", "Normal", "Thick"},
		},
		mode =
		{
			type = 'text',
			name = L["Mode"],
			desc = L["Choose what to show for health text"],
			get = function() return AloftHealthText.db.profile.mode end,
			set = function(value) AloftHealthText.db.profile.mode = value AloftHealthText:UpdateAll() end,
			validate = {"Percent", "Group Health", "Group Health Deficit", "Health"},
		},
		anchor = {
			type = "text", 
			name = L["Anchor"], 
			desc = L["Sets the anchor for the health text"],
			get = function() return AloftHealthText.db.profile.point end,
			set = function(v) AloftHealthText.db.profile.point = v AloftHealthText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		anchorto = {
			type = "text", 
			name = L["Anchor To"], 
			desc = L["Sets the relative point on the health bar to anchor the health text"],
			get = function() return AloftHealthText.db.profile.relativeToPoint end,
			set = function(v) AloftHealthText.db.profile.relativeToPoint = v AloftHealthText:UpdateAll() end,
			validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
		},
		offsetX =
		{
			type = 'range',
			name = L["X Offset"],
			desc = L["X offset of the health text"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftHealthText.db.profile.offsetX end,
			set = function(v) AloftHealthText.db.profile.offsetX = v AloftHealthText:UpdateAll() end
		},
		offsetY =
		{
			type = 'range',
			name = L["Y Offset"],
			desc = L["Y offset of the health text"],
			min = -32,
			max = 32,
			step = 1,
			get = function() return AloftHealthText.db.profile.offsetY end,
			set = function(v) AloftHealthText.db.profile.offsetY = v AloftHealthText:UpdateAll() end
		},
		alpha =
		{
			type = 'range',
			name = L["Alpha"],
			desc = L["Sets the alpha of the health text"],
			min = 0,
			max = 1.0,
			step = 0.05,
			get = function() return AloftHealthText.db.profile.alpha end,
			set = function(v) AloftHealthText.db.profile.alpha = v AloftHealthText:UpdateAll() end
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the health text color"],
			get = function() return unpack(AloftHealthText.db.profile.color) end,
			set = function(r, g, b, a) AloftHealthText.db.profile.color = { r, g, b, a } AloftHealthText:UpdateAll() end,
		},
	},
}

-----------------------------------------------------------------------------

local healthTextMethods =
{
	["Percent"] = function(frame, healthBar) return string.format("%d%%", floor(100 * frame.aloftData.healthBarValue / select(2, healthBar:GetMinMaxValues()))) end,
	["Group Health"] =	function(frame, healthBar) 
							local maxValue = select(2, healthBar:GetMinMaxValues())
							if maxValue == 100 then 
								return "" 
							end
							return Aloft:FormatLargeValue(frame.aloftData.healthBarValue)
						end,
	["Group Health Deficit"] =	function(frame, healthBar) 
									local maxValue = select(2, healthBar:GetMinMaxValues())
									if maxValue == 100 then 
										return "" 
									end
									local value = frame.aloftData.healthBarValue
									if value == maxValue then 
										return "" 
									end
									return Aloft:FormatLargeValue(maxValue - value)
								end,
	["Group Health"] =	function(frame, healthBar) 
							local maxValue = select(2, healthBar:GetMinMaxValues())
							if maxValue == 100 then 
								return ""
							end
							return Aloft:FormatLargeValue(frame.aloftData.healthBarValue)
						end,
	["Group Health Deficit"] =	function(frame, healthBar) 
									local maxValue = select(2, healthBar:GetMinMaxValues())
									if maxValue == 100 then 
										return "" 
									end
									local value = frame.aloftData.healthBarValue
									if value == maxValue then 
										return "" 
									end
									return Aloft:FormatLargeValue(maxValue - value)
								end,
	["Health"] =	function (frame, healthBar)
						local maxValue = select(2, healthBar:GetMinMaxValues())
						if maxValue ~= 100 then 
							return Aloft:FormatLargeValue(frame.aloftData.healthBarValue)
						end

						if MobHealth3 then
							local health, maxHealth, valid = MobHealth3:GetHealth(frame.aloftData.healthBarValue, maxValue, frame.aloftData.name, frame.aloftData.level)
							if valid then
								return Aloft:FormatLargeValue(health)
							end							
						elseif MobHealthDB then
							local mobHealthIndex = string.format("%s:%d", frame.aloftData.name, frame.aloftData.level)
							local mobHealthData = MobHealthDB[mobHealthIndex]
							if mobHealthData then
								local s, e, pts, pct = string.find(mobHealthData, "^(%d+)/(%d+)$")
								if( pts and pct ) then
									pct = tonumber(pct)
									if( pct ~= 0 ) then
										return Aloft:FormatLargeValue((frame.aloftData.healthBarValue * tonumber(pts) / pct))
									end
								end							
							end
						end
						
						return string.format("%d%%", frame.aloftData.healthBarValue)
					end
}

function AloftHealthText:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		if self.db.profile.enable then
			self:SetupFrame(frame)
			self:UpdateText(frame)
		else
			local healthTextFrame = frame.aloftData.healthTextFrame
			if healthTextFrame then
				healthTextFrame:Hide()
			end
		end
	end
	
	self:RegisterEvents()
end

-----------------------------------------------------------------------------

function AloftHealthText:RegisterEvents()
	if self.db.profile.enable then
		self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
		self:RegisterEvent("Aloft:OnHealthBarValueChanged", "UpdateText")
		self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
	else
		self:UnregisterAllEvents()
	end
end

function AloftHealthText:OnEnable()
	self:RegisterEvents()
end

function AloftHealthText:SetupFrame(frame)
	local healthTextFrame = frame.aloftData.healthTextFrame
	
	if healthTextFrame then
		healthTextFrame:Show()
	else
		healthTextFrame = frame:CreateFontString()
		frame.aloftData.healthTextFrame = healthTextFrame
	end
	
	self:PrepareText(healthTextFrame, self.db.profile)
	self:PlaceFrame(healthTextFrame, frame.aloftData.healthBar, self.db.profile)
end

function AloftHealthText:UpdateText(frame)
	local healthTextFrame = frame.aloftData.healthTextFrame
	healthTextFrame:SetText(healthTextMethods[self.db.profile.mode](frame, frame.aloftData.healthBar))
end

-----------------------------------------------------------------------------
